/*
 * Calculator State and Behavior
 *
 * Developed for "Rethinking CS101", a project of Lynn Andrea Stein's AP Group.
 * For more information, see <a href="http://www.ai.mit.edu/projects/cs101/">the
 * CS101 homepage</a> or email <las@ai.mit.edu>.
 *
 * Copyright (C) 1999 Massachusetts Institute of Technology.
 * Please do not redistribute without obtaining permission.
 */
package calculator;
import cs101.util.Coerce;
import cs101.io.Console;

/**
 * This class provides the main functionality for a basic four-function 
 * calculator.   Adapted from the earlier ButtonHandler<p>
 *
 * <P>Copyright (c) 1999 Massachusetts Institute of Technology
 *
 * @author  Todd C. Parnell, tparnell@ai.mit.edu
 * @author  Emil Sit, sit@mit.edu
 * @author  Lynn Andrea Stein, las@ai.mit.edu
 * @version $Id: CalculatorState.java,v 1.2 2003/03/28 17:25:06 gus Exp $
 *
 * @see Calculator
 */

public class CalculatorState extends Object
{
  /**
   * The GUI for which this CalculatorState is providing behavior.
   */
  private Calculator gui;

  protected boolean seenDecimal, readyForNewNumber;
  protected double previousNumber;
  protected int operation;
  
  protected CalculatorState ( Calculator gui )
  {
      super();
      this.gui = gui;
      this.resetCalc();
  }
  
  public String toString() 
  {
      return "CalculatorState:  currentNumber is " + this.readScreen() + ",\n" +
             "                  seenDecimal is " + this.seenDecimal + ",\n" +
             "                  operation is " + this.operation + ",\n" +
             "                  previousNumber is " + this.previousNumber + ",\n" +
             "                  readyForNewNumber is " + this.readyForNewNumber ;
  }

  protected void handleNumKey( int num ) 
  {
      if ( this.readyForNewNumber ) 
      {
          this.clearScreen();
          this.seenDecimal = false;
          this.readyForNewNumber = false;
      }
      this.writeScreen( this.readScreen() + String.valueOf(num) );
  }

  protected void handleDecimal () 
  {
      if ( this.readyForNewNumber ) 
      {
          this.handleNumKey( 0 );
      }
      if ( this.seenDecimal ) 
      {
          this.seenDecimal = false;
          this.writeScreen( this.readScreen() + ".");
      }
  }
	
  protected void handleOperation( int op ) 
  {
      this.handleEquals();
      this.operation = op;
      this.previousNumber = Coerce.stringToDouble( this.readScreen() );
  }

  protected void handleEquals () 
  {
      double answer = this.doOperation ( Coerce.stringToDouble( this.readScreen() ) );      

      this.writeScreen( answer );
      
      this.seenDecimal = true;
	  this.operation = Calculator.NO_OP;
      this.readyForNewNumber = true;
  }

  protected double doOperation ( double currentNumber ) {
    switch ( this.operation ) 
    {
      case Calculator.OP_MUL:
            return ( this.previousNumber * currentNumber );
      case Calculator.OP_DIV:
            return ( this.previousNumber / currentNumber );
      case Calculator.OP_SUB:
            return ( this.previousNumber - currentNumber );
      case Calculator.OP_ADD:
            return ( this.previousNumber + currentNumber );
      case Calculator.NO_OP:
           // fall through....
      default:
            return ( currentNumber );
    }
  }

  protected void resetDecimal () 
  {
    this.seenDecimal = false;
  }

  protected void clearScreen() 
  {
    this.writeScreen( "" );
  }
  
  protected String readScreen()
  {
  	return this.gui.getText();
  }
  
  protected void writeScreen( String s )
  {
      this.gui.setText( s );
  }
  
  protected void writeScreen( double d )
  {
      this.writeScreen( String.valueOf( d ) );
  }

  protected void writeScreen( int i )
  {
      this.writeScreen( String.valueOf( i ) );
  }

  protected void resetCalc() {
    this.clearScreen();
    this.writeScreen( 0.0 );
    this.previousNumber = 0.0;
    this.seenDecimal = true;
    this.readyForNewNumber = true;
    this.operation = Calculator.NO_OP;
  }

}


/* Comments:
 *
 * History:
 *     $Log: CalculatorState.java,v $
 *     Revision 1.2  2003/03/28 17:25:06  gus
 *     changed method name in Coerce
 *
 *     Revision 1.1.1.1  2002/06/05 21:56:34  root
 *     CS101 comes to Olin finally.
 *
 *     Revision 1.1  1999/10/08 15:09:24  las
 *     This pset replaces the old Calculator pset.  However, not everything
 *     has been transferred.  At the moment, it's just java, doc, and index.
 *     The rest are still in the repository under Calculator-Old.
 *
 */

